#ifndef BARRAY_H
#define BARRAY_H

#include "BeBuild.h"
#include "InterfaceDefs.h"
#include <malloc.h>
#include <memory.h>

template <class t>
class BArray {

	protected:

		t		*items;
		int		numItems;
		int		numSlots;
		int		blockSize;

inline	void		AssertSize(int size)
{
	if (size > numSlots) {
		numSlots = ((size+blockSize-1)/blockSize) * blockSize;
		items = (t*)realloc(items,numSlots*sizeof(t));
	};
};

	public:
inline			BArray(int _blockSize=64)
{
	blockSize = _blockSize;
	numItems = numSlots = 0;
	items = NULL;
};

inline			BArray(BArray<t> &copyFrom)
{
	blockSize = copyFrom.blockSize;
	numSlots = 0;
	items = NULL;
	AssertSize(copyFrom.numSlots);
	numItems = copyFrom.numItems;
	memcpy(items,copyFrom.items,numItems*sizeof(t));
};

inline			~BArray()
{
	if (items)
		free(items);
};

inline	t*		Items()
{
	return items;
};

inline	void		SetList(t* newList, int32 listSize)
{
	if (items)
		free(items);
	items = newList;
	numSlots = listSize;
	numItems = listSize;
};

inline	void		SetSlots(int32 slots)
{
	if (numSlots != slots) {
		numSlots = slots;
		if (numItems > numSlots)
			numItems = numSlots;
		items = (t*)realloc(items,numSlots*sizeof(t));
	};
};

inline	void		SetItems(int32 count)
{
	AssertSize(count);
	numItems = count;
};

inline	void		Trim()
{
	SetSlots(numItems);
};

inline	int32	CountItems() const
{ return numItems; };


inline	bool		IsMember(t &theT)
{
	for (int32 i=0;i<numItems;i++)
		if (theT == items[i]) return true;
	return false;
};

inline	void		RemoveItems(int32 index, int32 len)
{
	memmove(items+index,items+index+len,sizeof(t)*(numItems-index-len));
	numItems-=len;
};

inline	void		RemoveItem(int32 index)
{
	RemoveItems(index,1);
};

inline	int32	AddArray(BArray<t> *a)
{
	AssertSize(numItems + a->numItems);
	memcpy(items+numItems,a->items,a->numItems*sizeof(t));
	numItems = numItems + a->numItems;
	return a->numItems;
};

inline	int32	AddItem(const t &theT)
{
	AssertSize(numItems+1);
	items[numItems] = theT;
	numItems++;
	return numItems-1;
};

inline	void		MakeEmpty()
{
	numSlots = 0;
	numItems = 0;
	if (items!=NULL) {
		free(items);
		items = NULL;
	};
};

inline	t*		ItemAt(int index) const
				{ return( ( index<0 || index>=numItems ) ? NULL : &items[index] ); };

inline	t*		operator[](int index)
				{ return &items[index]; };
};

#endif
