#include "DumpApp.h"
#include "Candidate.h"

DumpApp::DumpApp() 
	: BApplication("application/x-vnd.f3c.dumper") {

	last_kill_time = system_time()+KILL_INTERVAL;
	supposed_to_stop = false;
	surveillance_thread = spawn_thread(surveil,"dumper surveillance", B_LOW_PRIORITY,this);
}

DumpApp::~DumpApp() {
}

void DumpApp::ReadyToRun() {
	resume_thread(surveillance_thread);
}

bool DumpApp::QuitRequested() {
	supposed_to_stop = true;
	status_t rc;
	wait_for_thread(surveillance_thread, &rc);
	return(BApplication::QuitRequested());
}

int32 DumpApp::surveil(void *data) {
	return(((DumpApp*)data)->Surveil());
}

int32 DumpApp::Surveil() {
	while (!supposed_to_stop) {
		CollectCandidates();
		snooze(10E6);
		CheckCandidates();
	}
	return(0);
}

void DumpApp::CollectCandidates() {
	app_info appInfo;
	thread_info threadInfo;
	int32 cookie=0;
	status_t rc;

	Lock();
	be_roster->GetAppInfo("application/x-vnd.Be-TRAK", &appInfo);
	while (get_next_thread_info(appInfo.team, &cookie, &threadInfo)==B_OK) {
		bool cand = true;
		if (!(threadInfo.name[0]=='w' && threadInfo.name[1]=='>')) cand = false;
		if (strcmp(threadInfo.name, "w>Desktop")==0) cand = false;
		for (int32 i=0; i<arrayCandidate.CountItems(); i++) {
			if (**arrayCandidate.ItemAt(i)==(&threadInfo)) cand = false;
		}
		
		if (cand) {
			arrayCandidate.AddItem(new Candidate(threadInfo));
		}
	}
	Unlock();
}

void DumpApp::CheckCandidates() {
	double rc, min=1E32;
	int32 min_i=-1;

	Lock();
	for (int32 i=0; i<arrayCandidate.CountItems(); i++) {
		rc = (*arrayCandidate.ItemAt(i))->Check();
		if (rc<min) {
			min = rc;
			min_i = i;
		}
	}
	if (min_i>=0&&(system_time()-last_kill_time>KILL_INTERVAL)) {
		Candidate *tmpCandidate = *arrayCandidate.ItemAt(min_i);
		if (tmpCandidate->GetAccu()<KILL_TRESHOLD) {
			tmpCandidate->Kill();
			arrayCandidate.RemoveItem(min_i);
			last_kill_time = system_time();
		}
	}
	Unlock();
}